@@ -199,6 +199,11 @@
 
 #undef HAVE_SOCKADDR_SA_LEN
 
+/* PAM (Pluggable Authentication Module) found? */
+#undef HAVE_PAM
+/* more Solaris bugs */
+#undef HAVE_SOLARIS_PAM_BUG
+
 /* interposition.c prototypes */
 #undef HAVE_PROT_CONNECT_0
 #undef HAVE_PROT_CONNECT_1
