@@ -41,7 +41,7 @@
  *
  */
 
-/* $Id: common.h,v 1.263 2001/03/05 13:55:51 michaels Exp $ */
+/* $Id: common.h,v 1.276 2001/05/12 13:16:24 karls Exp $ */
 
 #ifndef _COMMON_H_
 #define _COMMON_H_
@@ -74,10 +74,14 @@
 #include <sys/resource.h>
 #include <sys/ioctl.h>
 #include <sys/ipc.h>
+#if HAVE_SYS_SEM_H
 #include <sys/sem.h>
-/* XXX This is a hack. Avoid transparent sockaddr union used in Linux
-   to avoid the use of the union in the code. Mainly used in
-   interposition.c*/
+#endif /* HAVE_SYS_SEM_H */
+/*
+ * XXX This is a hack. Avoid transparent sockaddr union used in Linux
+ *  to avoid the use of the union in the code. Mainly used in
+ *  interposition.c
+ */
 #ifdef WE_DONT_WANT_NO_SOCKADDR_ARG_UNION
 #ifdef __GNUC__
 #define __HAD_GNUC __GNUC__
@@ -157,6 +161,9 @@
 #if HAVE_IFADDRS_H
 #include <ifaddrs.h>
 #endif /* HAVE_IFADDRS_H */
+#if HAVE_PAM
+#include <security/pam_appl.h> 
+#endif /* HAVE_PAM */
 
 #include "yacconfig.h"
 
@@ -372,9 +379,18 @@ struct ifaddrs {
 #define	ifa_broadaddr	ifa_dstaddr	/* broadcast address interface */
 #endif
 
-
 #endif /* !HAVE_GETIFADDRS */
 
+/*
+ * BSDI 4.1 doesn't have freeifaddrs(), but uses free()
+ *  Problem reported by "Zand, Nooshin" <nooshin.zand@intel.com>
+ */
+#if HAVE_GETIFADDRS && !HAVE_FREEIFADDRS
+#define freeifaddrs free
+#endif /* HAVE_GETIFADDRS && !HAVE_FREEIFADDRS */
+
+#define TOIN(addr) ((struct sockaddr_in *)(addr))
+#define TOCIN(addr) ((const struct sockaddr_in *)(addr))
 
 /* global variables needed by everyone. */
 extern struct config_t config;
@@ -422,7 +438,7 @@ extern int h_errno;
 #define	MAXRULEADDRSTRING	 (MAXSOCKSHOSTSTRING * 2)
 
 
-#define MAXAUTHINFOLEN		((sizeof("(") - 1) + MAXMETHODLEN) \
+#define MAXAUTHINFOLEN		((sizeof("(") - 1) + MAXMETHODSTRING) \
 									+ (sizeof(")" - 1) + (sizeof("@") - 1) + MAXNAMELEN)
 
 #ifndef NUL
@@ -714,16 +730,26 @@ do {														\
 #define AUTHMETHOD_NOACCEPT	255
 #define AUTHMETHOD_NOACCEPTs	"no acceptable method"
 
-/* not standard methods, must be > 255. */
-#define AUTHMETHOD_RFC931		256
+/* not standard methods, must be > AUTHMETHOD_NOACCEPT. */
+#define AUTHMETHOD_RFC931		(AUTHMETHOD_NOACCEPT + 1)
 #define AUTHMETHOD_RFC931s		"rfc931"
 
-#define AUTHMETHOD_MAX			(AUTHMETHOD_RFC931 + 1)
+#define AUTHMETHOD_PAM			(AUTHMETHOD_RFC931 + 1)
+#define AUTHMETHOD_PAMs			"pam"
 
-#define MAXMETHODLEN				MAX(sizeof(AUTHMETHOD_NONEs),		\
+#define AUTHMETHOD_MAX			(AUTHMETHOD_PAM + 1)
+
+#define MAXMETHODSTRING			MAX(sizeof(AUTHMETHOD_NONEs),		\
 										MAX(sizeof(AUTHMETHOD_GSSAPIs),	\
 										MAX(sizeof(AUTHMETHOD_UNAMEs),	\
-										sizeof(AUTHMETHOD_RFC931s))))
+										MAX(sizeof(AUTHMETHOD_RFC931s),	\
+										sizeof(AUTHMETHOD_PAMs)))))
+
+/* number of supported methods. */
+#define MAXMETHOD					1 /* NONE 		*/	\
+									 + 1 /* UNAME		*/	\
+									 + 1 /* RFC931		*/	\
+									 + 1 /* RFC931		*/
 
 /*
  *  Response commands/codes
@@ -1210,24 +1236,36 @@ struct udpheader_t {
 
 
 /* method username */
-struct uname_t {
+struct authmethod_uname_t {
 	unsigned char	version;
 	unsigned char	name[MAXNAMELEN];
 	unsigned char	password[MAXPWLEN];
 };
 
 /* method rfc931 */
-struct rfc931_t {
+struct authmethod_rfc931_t {
 	unsigned char	name[MAXNAMELEN];
 };
 
+/* method pam. */
+struct authmethod_pam_t {
+	char				servicename[MAXNAMELEN];	/* servicename to use with pam.	*/
+	unsigned char	name[MAXNAMELEN];
+	unsigned char	password[MAXPWLEN];
+};
+
 /* this must be big enough to hold a complete method request. */
 struct authmethod_t {
-	int						matched;	/* authentication matched?	*/
-	int						method;	/* method in use.				*/
+	int						method;					/* method in use.						*/
+	int						methodv[MAXMETHOD];	/* methods somewhere matched. 	*/
+	size_t					methodc;					/* number of methods matched.		*/
+	int						badmethodv[MAXMETHOD];/* methods not matched. 			*/
+	size_t					badmethodc;				/* number of methods not matched.*/
+
 	union {
-		struct uname_t		uname;
-		struct rfc931_t	rfc931;
+		struct authmethod_uname_t	uname;
+		struct authmethod_rfc931_t	rfc931;
+		struct authmethod_pam_t		pam;
 	} mdata;
 };
 
@@ -1264,8 +1302,8 @@ struct serverstate_t {
 	struct command_t			command;
 	struct extension_t		extension;
 	struct protocol_t			protocol;
-	int							methodv[AUTHMETHOD_MAX];/* methods to offer.			*/
-	int							methodc;						/* number of methods set.	*/
+	int							methodv[MAXMETHOD];		/* methods to offer.			*/
+	size_t						methodc;						/* number of methods set.	*/
 	struct proxyprotocol_t	proxyprotocol;
 };
 
@@ -1584,7 +1622,7 @@ ifname2sockaddr __P((const char *ifname, struct sockaddr *addr));
  * Returns:
  *		On success: "addr", filled in with the address found.
  *		On failure: NULL (no address found).
-*/
+ */
 
 int
 sockatmark __P((int s));
@@ -1621,7 +1659,7 @@ socks_recvfrom __P((int, void *, size_t, int, struct sockaddr *, socklen_t *,
 /*
  * Like recvfrom(), but with an additional auth argument to be used
  * if not NULL.
-*/
+ */
 
 ssize_t
 socks_sendto __P((int, const void *, size_t, int, const struct sockaddr *,
@@ -1629,7 +1667,7 @@ socks_sendto __P((int, const void *, size_t, int, const struct sockaddr *,
 /*
  * Like sendto(), but with an additional auth argument to be used
  * if not NULL.
-*/
+ */
 
 int
 closen __P((int));
@@ -1661,7 +1699,7 @@ snprintfn();
  * snprintf() but the following are also enforced:
  * 	returns 0 instead of -1 (rawterminates *str).
  *		never returns a value greater than size - 1.
-*/
+ */
 
 char *
 sockshost2string __P((const struct sockshost_t *host, char *string,
@@ -1865,10 +1903,10 @@ showstate __P((const struct serverstate_t *state));
  */
 
 void
-showmethod __P((int methodc, const int *methodv));
+showmethod __P((size_t methodc, const int *methodv));
 /*
  * Shows "methodv".
-*/
+ */
 
 struct route_t *
 addroute __P((const struct route_t *route));
@@ -2161,7 +2199,7 @@ socks_getpwnam __P((const char *login));
 /*
  * Like getpwnam() but works around sysv bug, tries to get the shadow
  * password too.
-*/
+ */
 
 __END_DECLS
 
