@@ -41,7 +41,7 @@
  *
  */
 
-/* $Id: config.h,v 1.43 2001/02/06 15:58:39 michaels Exp $ */
+/* $Id: config.h,v 1.45 2001/05/15 13:45:30 karls Exp $ */
 
 #ifndef _CONFIG_H_
 #define _CONFIG_H_
@@ -91,7 +91,7 @@
  * until we expire the badmarking so it will be tried again for new 
  * connections.
  * A value of zero means never.
-*/
+ */
 #define BADROUTE_EXPIRE				(60 * 0)
 
 	/*
@@ -107,6 +107,12 @@
 #define LIBWRAPBUF			80
 #endif  /* HAVE_LIBWRAP */
 
+/*
+ * Name to give as servicename when starting pam for rules that don't
+ * set it.
+ */
+#define DEFAULT_PAMSERVICENAME	"sockd"
+
 /*
  * used only if no usable system call is found (getdtablesize()/sysconf()).
  * If you are missing the system calls, but know what the value should
@@ -140,7 +146,7 @@
  *   It defaults to 511 instead of 512 because some systems store it
  *   as an 8-bit datatype; 512 truncated to 8-bits is 0, while 511 is
  *   255 when truncated.
-*/
+ */
 #define SOCKD_MAXCLIENTQUE			511
 
 
