@@ -41,7 +41,7 @@
  *
  */
 
-/* $Id: sockd.h,v 1.157 2001/02/06 15:58:40 michaels Exp $ */
+/* $Id: sockd.h,v 1.170 2001/05/14 11:47:25 michaels Exp $ */
 
 #ifndef _SOCKD_H_
 #define _SOCKD_H_
@@ -121,7 +121,6 @@ do {																			\
  * that was added to object on this call, or error.
 */
 
-
 #define SOCKD_NEWREQUEST	1	/* sending a new request	*/
 #define SOCKD_FREESLOT		2	/* free'd a slot.				*/
 
@@ -205,6 +204,10 @@ struct rule_t {
 	char							libwrap[LIBWRAPBUF];	/* libwrapline.					*/
 #endif  /* HAVE_LIBWRAP */
 
+#if HAVE_PAM
+	char							pamservicename[MAXNAMELEN];/* name for pamservice.	*/
+#endif /* HAVE_PAM */
+
 	struct rule_t				*next;			/* next rule in list.					*/
 };
 
@@ -236,6 +239,10 @@ struct userid_t {
 
 struct configstate_t {
 	unsigned						init:1;
+
+	/* allows us to optimize a few things a little based on configuration. */
+	unsigned						unfixedpamdata:1;		/* have rules with pamdata.*/
+
 #ifdef HAVE_VOLATILE_SIG_ATOMIC_T
 	sig_atomic_t				addchild;				/* okay to do a addchild()?	*/
 #else
@@ -299,8 +306,11 @@ struct config_t {
 	struct timeout_t				timeout;					/* timeout values.			*/
 	struct userid_t				uid;						/* userids.						*/
 
-	int								methodv[AUTHMETHOD_MAX];/* methods by priority.	*/
-	int								methodc;					/* methods in list.			*/
+	int								methodv[MAXMETHOD];  /* methods by priority.		*/
+	size_t							methodc;					/* methods in list.			*/
+
+	int								clientmethodv[MAXMETHOD]; /* clientmethods.		*/
+	size_t							clientmethodc;				  /* methods in list.	*/
 
 };
 
@@ -320,8 +330,8 @@ struct connectionstate_t {
 
 struct sockd_io_direction_t {
 	int								s;				/* socket connection.					*/
-	struct sockaddr_in			laddr;		/* local address of s.					*/
-	struct sockaddr_in			raddr;		/* address of remote peer for s.		*/
+	struct sockaddr				laddr;		/* local address of s.					*/
+	struct sockaddr				raddr;		/* address of remote peer for s.		*/
 
 	/*
 	 * Varies according to context.
@@ -348,7 +358,7 @@ struct sockd_io_t {
 
 	struct connectionstate_t		state;
 
-	struct sockd_io_direction_t	control;		/* control connection to client.	*/
+	struct sockd_io_direction_t	control;		/* clients controlconnection.		*/
 	struct sockd_io_direction_t	src;			/* client we receive data from.	*/
 	struct sockd_io_direction_t	dst;			/* remote peer.						*/
 
@@ -389,12 +399,12 @@ struct sockd_negotiate_t {
 
 
 struct sockd_request_t {
-	struct sockaddr_in			from;			/* client's control address.			*/
+	struct sockaddr				from;			/* client's control address.			*/
+	struct sockaddr				to;			/* address client was accepted on.	*/
 	struct request_t				req;			/* request to perform.					*/
 	struct rule_t					rule;			/* rule matched for accept().			*/
 	int								s;				/* clients control connection.		*/
 	struct connectionstate_t	state;		/* state of connection.					*/
-	struct sockaddr_in			to;			/* address client was accepted on.	*/
 };
 
 struct sockd_mother_t {
@@ -492,6 +502,12 @@ childtype2string __P((int type));
  * returns the string representation of "type".
  */
 
+const char *
+verdict2string __P((int verdict));
+/*
+ * returns the string representation of "verdict".
+ */
+
 int
 removechild __P((pid_t childpid));
 /*
@@ -565,13 +581,18 @@ authinfo __P((const struct authmethod_t *auth, char *info, size_t infolen));
 
 
 int
-rulespermit __P((int s, struct rule_t *rule,
-					  struct connectionstate_t *state,
-					  const struct sockshost_t *src, const struct sockshost_t *dst));
+rulespermit __P((int s,
+						const struct sockaddr *peer, const struct sockaddr *local,
+					  struct rule_t *rule, struct connectionstate_t *state,
+					  const struct sockshost_t *src, const struct sockshost_t *dst,
+					  char *msg, size_t msgsize));
 /*
  * Checks whether the rules permit data from "src" to "dst".
- * "s" is the socket the connection is on.
+ * "s" is the socket the connection is on, from the address "peer", accepted
+ * on the address "local".
  * "state" is the current state of the connection and may be updated.
+ * "msg" is filled in with any message/information provided when checking
+ * access, "msgsize" is the size of "msg".
  *
  * Wildcard fields are supported for the following fields;
  *		ipv4:			INADDR_ANY
@@ -646,11 +667,13 @@ send_client __P((int s, int client));
  */
 
 int
-selectmethod __P((const unsigned char *methodv, size_t methodc));
+selectmethod __P((const int *methodv, size_t methodc,
+					   const unsigned char *offeredv, size_t offeredc));
 /*
  * Selects the best method based on available methods and given
- * priority.  "methodv" is a list over available methods, methodc
- * in length.
+ * priority.
+ * "methodv" is a list over available methods, methodc in length.
+ * "offerdv" is a list over offered methods, offeredc in length.
  * The function returns the value of the method that should be selected,
  * AUTMETHOD_NOACCEPT if none is acceptable.
  */
@@ -709,19 +732,6 @@ iolog __P((struct rule_t *rule, const struct connectionstate_t *state,
  */
 
 
-int
-serverchild __P((int s, const struct sockaddr_in *local,
-					  const struct sockaddr_in *remote));
-/*
- * Forked off by the server after a connection has come in on "s" but
- * before any data has been read.  "local" is the address "remote"
- * connected to.
- * Returns:
- *		On success: 0
- *		On failure : -1
- */
-
-
 void
 close_iodescriptors __P((const struct sockd_io_t *io));
 /*
@@ -924,7 +934,31 @@ socks_reseteuid __P((uid_t current, uid_t new));
  */
 
 int
-passwordcheck __P((const char *name, const char *cleartextpassword));
+accessmatch __P((int s, struct authmethod_t *auth,
+					  const struct sockaddr *src, const struct sockaddr *dst,
+					  const struct linkedname_t *userlist, char *emsg,
+					  size_t emsgsize));
+/*
+ * Checks whether access matches according to supplied arguements.
+ * "auth" is the authentication to be matched against, 
+ * "s" is the socket the client is connected to,
+ * "src" is address client connected from, "dst" is address client
+ * connected to.
+ * "userlist", if not NULL, is a list of names that restricts access
+ * further, the authentication must in this case also result in a 
+ * username present in "userlist".
+ * "emsg" is a buffer that information can be written into, "emsgsize"
+ * is the size of that buffer.
+ *
+ * Returns:
+ *		If access is ok: true.
+ * 	Otherwise: false.  Writes the reason into "emsg".
+ */
+
+
+int
+passwordcheck __P((const char *name, const char *cleartextpassword, 
+						 char *emsg, size_t emsglen));
 /*
  * Checks whether "name" is in the passwordfile.
  * If "cleartextpassword" is not NULL, also checks if "name"'s
@@ -932,10 +966,25 @@ passwordcheck __P((const char *name, const char *cleartextpassword));
  *
  * Returns:
  *		If "name" and "cleartextpassword" is matched: 0
- *		If "name" is unknown: 1
- *		If "cleartextpassword" does not match: 2
+ *		Otherwise: -1.  "emsg" is filled in with the errormessage.
  */
 
+int
+pam_passwordcheck __P((int s,
+							  const struct sockaddr *src, const struct sockaddr *dst,
+							  const struct authmethod_pam_t *auth, char *emsg,
+							  size_t emsglen));
+/*
+ * Checks whether pam grants access to the client connected to the socket "s".
+ * "src" is the clients sourceaddress, "dst" is address we accepted the
+ * clients connection on.
+ *
+ * Returns:
+ *		If "name" and "cleartextpassword" is matched: 0
+ *		Otherwise: -1.  "emsg" is filled in with the errormessage.
+ */
+
+
 #ifdef DEBUG
 void
 printfd __P((const struct sockd_io_t *io, const char *prefix));
