@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,8 +32,8 @@
  *  Software Distribution Coordinator  or  sdc@inet.no
  *  Inferno Nettverk A/S
  *  Oslo Research Park
- *  Gaustadalen 21
- *  N-0349 Oslo
+ *  Gaustadalllllen 21
+ *  NO-0349 Oslo
  *  Norway
  *
  * any improvements or extensions that they make and grant Inferno Nettverk A/S
@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: Rgetpeername.c,v 1.24 1999/05/14 14:44:38 michaels Exp $";
+"$Id: Rgetpeername.c,v 1.27 2001/02/06 15:58:48 michaels Exp $";
 
 int
 Rgetpeername(s, name, namelen)
@@ -69,8 +69,8 @@ Rgetpeername(s, name, namelen)
 			break;
 
 		case SOCKS_CONNECT:
-			if (socksfd->state.inprogress) {
-				errno = EINPROGRESS;
+			if (socksfd->state.err != 0) {
+				errno = ENOTCONN;
 				return -1;
 			}
 			addr = &socksfd->connected;
