@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: authneg.c,v 1.50 2001/02/06 15:58:50 michaels Exp $";
+"$Id: authneg.c,v 1.51 2001/04/25 11:31:53 michaels Exp $";
 
 int
 negotiate_method(s, packet)
@@ -71,7 +71,7 @@ negotiate_method(s, packet)
 	/* create request packet. */
 	request[AUTH_VERSION]	= packet->req.version;
 	request[AUTH_NMETHODS]	= (unsigned char)packet->gw.state.methodc;
-	for (rc = 0; rc < packet->gw.state.methodc; ++rc)
+	for (rc = 0; rc < (int)packet->gw.state.methodc; ++rc)
 		request[AUTH_METHODS + rc] = (unsigned char)packet->gw.state.methodv[rc];
 
 	/* send list over methods we support */
