@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: udp.c,v 1.119 2001/02/06 15:58:59 michaels Exp $";
+"$Id: udp.c,v 1.121 2001/05/02 11:37:20 michaels Exp $";
 
 /* ARGSUSED */
 ssize_t
@@ -56,8 +56,10 @@ Rsendto(s, msg, len, flags, to, tolen)
 	const struct sockaddr *to;
 	socklen_t tolen;
 {
+	const char *function = "Rsendto()";
 	struct socksfd_t *socksfd;
 	struct sockshost_t host;
+	char srcstring[MAXSOCKADDRSTRING], dststring[sizeof(srcstring)];
 	char *nmsg;
 	size_t nlen;
 	ssize_t n;
@@ -74,9 +76,16 @@ Rsendto(s, msg, len, flags, to, tolen)
 	if (to == NULL)
 		if (socksfd->state.udpconnect)
 			to = &socksfd->connected;
-		else
-			/* best we can do. */
-			return sendto(s, msg, len, flags, NULL, 0);
+		else { /* tcp. */
+			n =  sendto(s, msg, len, flags, NULL, 0);
+
+			slog(LOG_DEBUG, "%s: %s: %s -> %s (%lu)", 
+			function, protocol2string(SOCKS_TCP),
+			sockaddr2string(&socksfd->local, dststring, sizeof(dststring)),
+			sockaddr2string(&socksfd->server, srcstring, sizeof(srcstring)), n);
+
+			return n;
+		}
 
 	/* prefix a UDP header to the msg */
 	nlen = len;
@@ -94,6 +103,11 @@ Rsendto(s, msg, len, flags, to, tolen)
 
 	free(nmsg);
 
+	slog(LOG_DEBUG, "%s: %s: %s -> %s (%lu)", 
+	function, protocol2string(SOCKS_TCP),
+	sockaddr2string(&socksfd->local, dststring, sizeof(dststring)),
+	sockaddr2string(&socksfd->reply, srcstring, sizeof(srcstring)), n);
+
 	return MAX(-1, n);
 }
 
@@ -109,9 +123,10 @@ Rrecvfrom(s, buf, len, flags, from, fromlen)
 	const char *function = "Rrecvfrom()";
 	struct socksfd_t *socksfd;
 	struct udpheader_t header;
-	char *newbuf;
 	struct sockaddr newfrom;
+	char srcstring[MAXSOCKADDRSTRING], dststring[sizeof(srcstring)];
 	socklen_t newfromlen;
+	char *newbuf;
 	size_t newlen;
 	ssize_t n;
 
@@ -136,10 +151,18 @@ Rrecvfrom(s, buf, len, flags, from, fromlen)
 				errno = ENOTCONN;
 				return -1;
 			}
-		return recvfrom(s, buf, len, flags, from, fromlen);
+
+		n = recvfrom(s, buf, len, flags, from, fromlen);
+
+		slog(LOG_DEBUG, "%s: %s: %s -> %s (%lu)", 
+		function, protocol2string(SOCKS_TCP),
+		sockaddr2string(&socksfd->connected, srcstring, sizeof(srcstring)),
+		sockaddr2string(&socksfd->local, dststring, sizeof(dststring)), n);
+		
+		return n;
 	}
 
-	/* if packet is from socksserver it will be prefixed with a header. */
+	/* udp.  If packet is from socksserver it will be prefixed with a header. */
 	newlen = len + sizeof(header);
 	if ((newbuf = (char *)malloc(sizeof(*newbuf) * newlen)) == NULL) {
 		errno = ENOBUFS;
@@ -231,6 +254,11 @@ Rrecvfrom(s, buf, len, flags, from, fromlen)
 
 	free(newbuf);
 
+	slog(LOG_DEBUG, "%s: %s: %s -> %s (%lu)", 
+	function, protocol2string(SOCKS_UDP), 
+	sockaddr2string(&newfrom, srcstring, sizeof(srcstring)),
+	sockaddr2string(&socksfd->local, dststring, sizeof(dststring)), n);
+
 	if (from != NULL) {
 		*fromlen = MIN(*fromlen, newfromlen);
 		memcpy(from, &newfrom, (size_t)*fromlen);
@@ -323,17 +351,16 @@ udpsetup(s, to, type)
 	}
 
 	/* LINTED  pointer casts may be troublesome */
-	if ((((struct sockaddr_in *)(&socksfd.local))->sin_addr.s_addr
-	== htonl(INADDR_ANY))
+	if ((TOIN((&socksfd.local))->sin_addr.s_addr == htonl(INADDR_ANY))
 	/* LINTED  pointer casts may be troublesome */
-	|| ((struct sockaddr_in *)(&socksfd.local))->sin_port == htons(0)) {
+	|| TOIN((&socksfd.local))->sin_port == htons(0)) {
 		/*
 		 * local name not fixed, set it, port may be bound, we need to bind
 		 * IP too however.
 		 */
 
 		/* LINTED  pointer casts may be troublesome */
-		const in_port_t port = ((struct sockaddr_in *)(&socksfd.local))->sin_port;
+		const in_port_t port = TOIN((&socksfd.local))->sin_port;
 
 		if (port != htons(0)) {
 			/*
@@ -364,7 +391,7 @@ udpsetup(s, to, type)
 			return -1;
 		}
 		/* LINTED  pointer casts may be troublesome */
-		((struct sockaddr_in *)&socksfd.local)->sin_port = port;
+		TOIN(&socksfd.local)->sin_port = port;
 
 		if (bind(s, &socksfd.local, sizeof(socksfd.local)) != 0) {
 			close(socksfd.control);
