@@ -51,7 +51,7 @@
 #endif  /* HAVE_STRVIS */
 
 static const char rcsid[] =
-"$Id: util.c,v 1.113 2001/02/06 15:59:00 michaels Exp $";
+"$Id: util.c,v 1.121 2001/05/15 14:45:46 michaels Exp $";
 
 /* fake "ip address", for clients without DNS access. */
 static char **ipv;
@@ -162,6 +162,9 @@ method2string(method)
 		case AUTHMETHOD_RFC931:
 			return AUTHMETHOD_RFC931s;
 
+		case AUTHMETHOD_PAM:
+			return AUTHMETHOD_PAMs;
+
 		default:
 			SERRX(method);
 	}
@@ -179,7 +182,8 @@ string2method(methodname)
 	} method[] = {
 		{ AUTHMETHOD_NONEs,		AUTHMETHOD_NONE	},
 		{ AUTHMETHOD_UNAMEs,		AUTHMETHOD_UNAME	},
-		{ AUTHMETHOD_RFC931s,	AUTHMETHOD_RFC931	}
+		{ AUTHMETHOD_RFC931s,	AUTHMETHOD_RFC931	},
+		{ AUTHMETHOD_PAMs,		AUTHMETHOD_PAM		}
 	};
 	size_t i;
 
@@ -279,20 +283,25 @@ sockshost2sockaddr(host, addr)
 	struct sockaddr *addr;
 {
 	const char *function = "sockshost2sockaddr()";
+	uint8_t sa_len;
 
 	bzero(addr, sizeof(*addr));
-	/* LINTED pointer casts may be troublesome */
-	((struct sockaddr_in *)addr)->sin_family = AF_INET;
 
 	switch (host->atype) {
 		case SOCKS_ADDR_IPV4:
+			addr->sa_family = AF_INET;
+			sa_len = sizeof(struct sockaddr_in);
+
 			/* LINTED pointer casts may be troublesome */
-			((struct sockaddr_in *)addr)->sin_addr = host->addr.ipv4;
+			TOIN(addr)->sin_addr = host->addr.ipv4;
 			break;
 
 		case SOCKS_ADDR_DOMAIN: {
 			struct hostent *hostent;
 
+			addr->sa_family = AF_INET;
+			sa_len = sizeof(struct sockaddr_in);
+
 			if ((hostent = gethostbyname(host->addr.domain)) == NULL
 			||   hostent->h_addr_list == NULL) {
 				/* LINTED pointer casts may be troublesome */
@@ -300,14 +309,13 @@ sockshost2sockaddr(host, addr)
 				function, host->addr.domain, hstrerror(h_errno));
 
 				/* LINTED pointer casts may be troublesome */
-				((struct sockaddr_in *)addr)->sin_addr.s_addr = htonl(INADDR_ANY);
+				TOIN(addr)->sin_addr.s_addr = htonl(INADDR_ANY);
 
 				break;
 			}
 
 			/* LINTED pointer casts may be troublesome */
-			((struct sockaddr_in *)addr)->sin_addr
-			= *(struct in_addr *)(*hostent->h_addr_list);
+			TOIN(addr)->sin_addr = *(struct in_addr *)(*hostent->h_addr_list);
 
 			break;
 		}
@@ -315,10 +323,14 @@ sockshost2sockaddr(host, addr)
 		default:
 			SERRX(host->atype);
 	}
-	/* LINTED pointer casts may be troublesome */
-	((struct sockaddr_in *)addr)->sin_port = host->port;
+
+#if HAVE_SOCKADDR_SA_LEN
+	addr->sa_len = sa_len;
+#endif
 
 	/* LINTED pointer casts may be troublesome */
+	TOIN(addr)->sin_port = host->port;
+
 	return addr;
 }
 
@@ -328,32 +340,38 @@ fakesockshost2sockaddr(host, addr)
 	struct sockaddr *addr;
 {
 	const char *function = "fakesockshost2sockaddr()";
+	char string[MAXSOCKSHOSTSTRING];
+	uint8_t sa_len;
 
 #if SOCKS_CLIENT /* may be called before normal init, log to right place. */
 	clientinit();
 #endif
 
-	slog(LOG_DEBUG, "%s: %s", function, sockshost2string(host, NULL, 0));
+	slog(LOG_DEBUG, "%s: %s",
+	function, sockshost2string(host, string, sizeof(string)));
 
 	bzero(addr, sizeof(*addr));
-	/* LINTED pointer casts may be troublesome */
-	((struct sockaddr_in *)addr)->sin_family = AF_INET;
 
 	switch (host->atype) {
 		case SOCKS_ADDR_DOMAIN:
+			addr->sa_family = AF_INET;
+			sa_len = sizeof(struct sockaddr_in);
+
 			/* LINTED pointer casts may be troublesome */
-			if (socks_getfakeip(host->addr.domain,
-			&((struct sockaddr_in *)addr)->sin_addr)) {
-				/* LINTED pointer casts may be troublesome */
+			if (socks_getfakeip(host->addr.domain, &TOIN(addr)->sin_addr))
 				break;
-			}
 			/* else; */ /* FALLTHROUGH */
 
 		default:
 			return sockshost2sockaddr(host, addr);
 	}
+
+#if HAVE_SOCKADDR_SA_LEN
+	addr->sa_len = sa_len;
+#endif
+
 	/* LINTED pointer casts may be troublesome */
-	((struct sockaddr_in *)addr)->sin_port = host->port;
+	TOIN(addr)->sin_port = host->port;
 
 	return addr;
 }
@@ -368,9 +386,9 @@ sockaddr2sockshost(addr, host)
 		case AF_INET:
 			host->atype			= SOCKS_ADDR_IPV4;
 			/* LINTED pointer casts may be troublesome */
-			host->addr.ipv4	= ((const struct sockaddr_in *)addr)->sin_addr;
+			host->addr.ipv4	= TOCIN(addr)->sin_addr;
 			/* LINTED pointer casts may be troublesome */
-			host->port			= ((const struct sockaddr_in *)addr)->sin_port;
+			host->port			= TOCIN(addr)->sin_port;
 			break;
 
 		default:
@@ -597,7 +615,7 @@ ifname2sockaddr(ifname, addr)
 		if (strcmp(iface->ifa_name, ifname) == 0 
 		&& iface->ifa_addr != NULL && iface->ifa_addr->sa_family == AF_INET) { 
 			*addr = *iface->ifa_addr;
-			free(ifap); 
+			freeifaddrs(ifap); 
 			return addr;
 		}
 
@@ -652,7 +670,7 @@ sockaddr2string(address, string, len)
 
 		case AF_INET: {
 			/* LINTED pointer casts may be troublesome */
-			const struct sockaddr_in *addr = (const struct sockaddr_in *)address;
+			const struct sockaddr_in *addr = TOCIN(address);
 
 			snprintfn(string, len, "%s.%d",
 			inet_ntoa(addr->sin_addr), ntohs(addr->sin_port));
@@ -732,6 +750,7 @@ fakesockaddr2sockshost(addr, host)
 	struct sockshost_t *host;
 {
 	const char *function = "fakesockaddr2sockshost()";
+	char string[MAXSOCKADDRSTRING];
 
 #if SOCKS_CLIENT /* may be called before normal init, log to right place. */
 	clientinit();
@@ -739,17 +758,14 @@ fakesockaddr2sockshost(addr, host)
 
 	/* LINTED pointer casts may be troublesome */
 	slog(LOG_DEBUG, "%s: %s -> %s",
-	function, sockaddr2string(addr, NULL, 0),
-	socks_getfakehost(((const struct sockaddr_in *)addr)->sin_addr.s_addr)
-	== NULL ? sockaddr2string(addr, NULL, 0)
-	: socks_getfakehost(((const struct sockaddr_in *)addr)->sin_addr.s_addr));
+	function, sockaddr2string(addr, string, sizeof(string)),
+	socks_getfakehost(TOCIN(addr)->sin_addr.s_addr) == NULL ?
+	string : socks_getfakehost(TOCIN(addr)->sin_addr.s_addr));
 
 	/* LINTED pointer casts may be troublesome */
-	if (socks_getfakehost(((const struct sockaddr_in *)addr)->sin_addr.s_addr)
-	!= NULL) {
-		const char *ipname
+	if (socks_getfakehost(TOCIN(addr)->sin_addr.s_addr) != NULL) {
 		/* LINTED pointer casts may be troublesome */
-		= socks_getfakehost(((const struct sockaddr_in *)addr)->sin_addr.s_addr);
+		const char *ipname = socks_getfakehost(TOCIN(addr)->sin_addr.s_addr);
 
 		SASSERTX(ipname != NULL);
 
@@ -757,7 +773,7 @@ fakesockaddr2sockshost(addr, host)
 		SASSERTX(strlen(ipname) < sizeof(host->addr.domain));
 		strcpy(host->addr.domain, ipname);
 		/* LINTED pointer casts may be troublesome */
-		host->port	= ((const struct sockaddr_in *)addr)->sin_port;
+		host->port	= TOCIN(addr)->sin_port;
 	}
 	else
 		sockaddr2sockshost(addr, host);
@@ -863,9 +879,9 @@ sockaddrareeq(a, b)
 	switch (a->sa_family) {
 		case AF_INET: {
 			/* LINTED pointer casts may be troublesome */
-			const struct sockaddr_in *in_a = (const struct sockaddr_in *)a;
+			const struct sockaddr_in *in_a = TOCIN(a);
 			/* LINTED pointer casts may be troublesome */
-			const struct sockaddr_in *in_b = (const struct sockaddr_in *)b;
+			const struct sockaddr_in *in_b = TOCIN(b);
 
 			if (in_a->sin_addr.s_addr != in_b->sin_addr.s_addr
 			||  in_a->sin_port		  != in_b->sin_port)
@@ -971,7 +987,7 @@ socketoptdup(s)
 {
 	const char *function = "socketoptdup()";
 	unsigned int i;
-	int flags, new_s;
+	int flags, new_s, errno_s;
 	socklen_t len;
 	union {
 		int					int_val;
@@ -1112,6 +1128,8 @@ socketoptdup(s)
 
 	};
 
+	errno_s = errno;
+
 	len = sizeof(val);
 	if (getsockopt(s, SOL_SOCKET, SO_TYPE, &val, &len) == -1) {
 		swarn("%s: getsockopt(SO_TYPE)", function);
@@ -1149,6 +1167,8 @@ socketoptdup(s)
 		swarn("%s: fcntl(F_GETFD/F_SETFD)", function);
 #endif
 
+	errno = errno_s;
+
 	return new_s;
 }
 
