@@ -45,7 +45,7 @@
 #include "config_parse.h"
 
 static const char rcsid[] =
-"$Id: serverconfig.c,v 1.130 2001/03/13 21:40:14 michaels Exp $";
+"$Id: serverconfig.c,v 1.153 2001/05/14 11:47:27 michaels Exp $";
 
 __BEGIN_DECLS
 
@@ -162,8 +162,7 @@ addinternal(addr)
 						NEWINTERNAL_EXTERNAL(config.internalc, config.internalv);
 
 						/* LINTED pointer casts may be troublesome */
-						((struct sockaddr_in *)iface->ifa_addr)->sin_port
-						= addr->port.tcp;
+						TOIN(iface->ifa_addr)->sin_port = addr->port.tcp;
 
 						config.internalv[config.internalc - 1].addr
 						= *iface->ifa_addr;
@@ -210,16 +209,21 @@ addclientrule(newrule)
 	const struct rule_t *newrule;
 {
 	struct rule_t *rule, ruletoadd;
-	int i;
+	size_t i;
 
 	ruletoadd = *newrule; /* for const. */
 
 	rule = addrule(&ruletoadd, &config.crule, 1);
 
+	if (rule->state.methodc == 0)
+		if (rule->user == NULL)
+			rule->state.methodv[rule->state.methodc++] = AUTHMETHOD_NONE;
+		
 	for (i = 0; i < rule->state.methodc; ++i)
 		switch (rule->state.methodv[i]) {
 			case AUTHMETHOD_NONE:
 			case AUTHMETHOD_RFC931:
+			case AUTHMETHOD_PAM:
 				break;
 
 			default:
@@ -227,19 +231,6 @@ addclientrule(newrule)
 				method2string(rule->state.methodv[i]));
 		}
 
-	if (rule->user != NULL) {
-		/* check that all methods given handle usernames. */
-		for (i = 0; i < rule->state.methodc; ++i)
-			switch (rule->state.methodv[i]) {
-				case AUTHMETHOD_RFC931:
-					break;
-
-				default:
-					yyerror("method \"%s\" can not provide usernames",
-					method2string(rule->state.methodv[i]));
-			}
-	}
-
 	checkrule(rule);
 
 	/* LINTED cast discards 'const' from pointer target type */
@@ -296,8 +287,7 @@ showrule(rule)
 	slog(LOG_INFO, "socks-rule #%u, line #%lu",
 	rule->number, rule->linenumber);
 
-	slog(LOG_INFO, "verdict: %s",
-	rule->verdict == VERDICT_PASS ? VERDICT_PASSs : VERDICT_BLOCKs);
+	slog(LOG_INFO, "verdict: %s", verdict2string(rule->verdict));
 
 	slog(LOG_INFO, "src: %s",
 	ruleaddress2string(&rule->src, addr, sizeof(addr)));
@@ -309,6 +299,12 @@ showrule(rule)
 
 	showstate(&rule->state);
 
+#if HAVE_PAM
+	if (*rule->pamservicename != NUL)
+		slog(LOG_INFO, "pamservicename: %s", rule->pamservicename);
+#endif  /* HAVE_PAM */
+
+
 	showlog(&rule->log);
 
 #if HAVE_LIBWRAP
@@ -326,8 +322,7 @@ showclient(rule)
 	slog(LOG_INFO, "client-rule #%u, line #%lu",
 	rule->number, rule->linenumber);
 
-	slog(LOG_INFO, "verdict: %s",
-	rule->verdict == VERDICT_PASS ? VERDICT_PASSs : VERDICT_BLOCKs);
+	slog(LOG_INFO, "verdict: %s", verdict2string(rule->verdict));
 
 	slog(LOG_INFO, "src: %s",
 	ruleaddress2string(&rule->src, addr, sizeof(addr)));
@@ -337,6 +332,11 @@ showclient(rule)
 
 	showmethod(rule->state.methodc, rule->state.methodv);
 
+#if HAVE_PAM
+	if (*rule->pamservicename != NUL)
+		slog(LOG_INFO, "pamservicename: %s", rule->pamservicename);
+#endif  /* HAVE_PAM */
+
 	showuser(rule->user);
 
 	showlog(&rule->log);
@@ -436,11 +436,17 @@ showconfig(config)
 	(unsigned long)config->uid.libwrap);
 
 	bufused = snprintfn(buf, sizeof(buf), "method(s): ");
-	for (i = 0; i < config->methodc; ++i)
+	for (i = 0; (size_t)i < config->methodc; ++i)
 		bufused += snprintfn(&buf[bufused], sizeof(buf) - bufused, "%s%s",
 		i > 0 ? ", " : "", method2string(config->methodv[i]));
 	slog(LOG_INFO, buf);
 
+	bufused = snprintfn(buf, sizeof(buf), "clientmethod(s): ");
+	for (i = 0; (size_t)i < config->clientmethodc; ++i)
+		bufused += snprintfn(&buf[bufused], sizeof(buf) - bufused, "%s%s",
+		i > 0 ? ", " : "", method2string(config->clientmethodv[i]));
+	slog(LOG_INFO, buf);
+
 	if (config->option.debug) {
 		struct rule_t *rule;
 		int count;
@@ -536,6 +542,10 @@ resetconfig(void)
 	bzero(config.methodv, sizeof(config.methodv));
 	config.methodc = 0;
 
+	bzero(config.clientmethodv, sizeof(config.clientmethodv));
+	config.clientmethodc = 0;
+
+
 	/* timeout, read from configfile. */
 	bzero(&config.timeout, sizeof(config.timeout));
 
@@ -579,7 +589,7 @@ iolog(rule, state, operation, src, srcauth, dst, dstauth, data, count)
 	sockshost2string(dst, &dststring[p], sizeof(dststring) - p);
 
 	snprintfn(rulecommand, sizeof(rulecommand), "%s(%d): %s/%s", 
-	rule->verdict == VERDICT_PASS ? VERDICT_PASSs : VERDICT_BLOCKs,
+	verdict2string(rule->verdict),
 	rule->number, protocol2string(state->protocol),
 	command2string(state->command));
 
@@ -587,7 +597,10 @@ iolog(rule, state, operation, src, srcauth, dst, dstauth, data, count)
 		case OPERATION_ACCEPT:
 		case OPERATION_CONNECT:
 			if (rule->log.connect)
-				slog(LOG_INFO, "%s [: %s -> %s", rulecommand, srcstring, dststring);
+				slog(LOG_INFO, "%s [: %s -> %s%s%s",
+				rulecommand, srcstring, dststring,
+				(data == NULL || *data == NUL) ? "" : ": ",
+				(data == NULL || *data == NUL) ? "" : data);
 			break;
 
 		case OPERATION_ABORT:
@@ -629,18 +642,23 @@ iolog(rule, state, operation, src, srcauth, dst, dstauth, data, count)
 }
 
 int
-rulespermit(s, match, state, src, dst)
+rulespermit(s, peer, local, match, state, src, dst, msg, msgsize)
 	int s;
+	const struct sockaddr *peer, *local;
 	struct rule_t *match;
 	struct connectionstate_t *state;
 	const struct sockshost_t *src;
 	const struct sockshost_t *dst;
+	char *msg;
+	size_t msgsize;
 {
 	const char *function = "rulespermit()";
 	static int init;
 	static struct rule_t defrule;
 	struct rule_t *rule;
 	struct connectionstate_t ostate;
+	int *methodv;
+	int methodc;
 #if HAVE_LIBWRAP
 	struct request_info libwraprequest;
 
@@ -690,22 +708,34 @@ rulespermit(s, match, state, src, dst)
 		init = 1;
 	}
 
+	if (msgsize > 0)
+		*msg = NUL;
+
 	/* what rulebase to use.  XXX nicer way to do this. */
 	switch (state->command) {
 		case SOCKS_ACCEPT:
 			/* only set by negotiate children so must be clientrule. */
-			rule = config.crule;
+			rule 		= config.crule;
+			methodv	= config.clientmethodv;
+			methodc	= config.clientmethodc;
 			break;
 
 		default:
 			/* everyone else, socksrules. */
 			rule = config.srule;
+			methodv	= config.methodv;
+			methodc	= config.methodc;
 			break;
 	}
 
-	/* let "state" be unchanged from original unless we actually get a match. */
+	/*
+	 * let "state" be unchanged from original unless we actually get a match.
+	 * The exception to this is state->auth.methodv and state->auth.badmethodv,
+	 * we change them so we can "cache" it, and callee could in theory
+	 * use it to see which methods we tried.
+	 */
 	for (ostate = *state; rule != NULL; rule = rule->next, *state = ostate) {
-		char *name, *password;
+		int i;
 
 		/* current rule covers desired command? */
 		switch (state->command) {
@@ -778,88 +808,135 @@ rulespermit(s, match, state, src, dst)
 
 		/* current rule allows for selected authentication? */
 		if (!methodisset(state->auth.method, rule->state.methodv,
-		(size_t)rule->state.methodc))
+		rule->state.methodc)) {
+			size_t methodisok;
+
 			/*
-			 * There are some "extra" methods that are independent of
-			 * socks protocol negotiation and it's thus possible
-			 * to get a match on them even if above check failed.
-			 * Currently it's only rfc931.
+			 * There are some "extra" (non-standard) methods that are independent
+			 * of socks protocol negotiation and it's thus possible
+			 * to get a match on them even if above check failed, i.e.
+			 * it's possible to "upgrade" the method. 
+			 * 
+			 * We therefor look at what methods this rule wants and see
+			 * if can match it with what we have, or get it.
+			 *
+			 * Currently these methods are: rfc931 and pam.
 			 */
 
+			for (i = methodisok = 0; i < methodc; ++i) {
+				if (methodisset(methodv[i], rule->state.methodv,
+				rule->state.methodc)) {
+					switch (methodv[i]) {
 #if HAVE_LIBWRAP
-			if (methodisset(AUTHMETHOD_RFC931, rule->state.methodv,
-			(size_t)rule->state.methodc)) {
-				strncpy((char *)state->auth.mdata.rfc931.name,
-				eval_user(&libwraprequest),
-				sizeof(state->auth.mdata.rfc931.name) - 1);
+						case AUTHMETHOD_RFC931:
+							strncpy((char *)state->auth.mdata.rfc931.name,
+							eval_user(&libwraprequest),
+							sizeof(state->auth.mdata.rfc931.name) - 1);
 
-				/* libwrap sets it to unknown if no identreply. */
-				if (strcmp((char *)state->auth.mdata.rfc931.name, STRING_UNKNOWN)
-				== 0)
-					*state->auth.mdata.rfc931.name = NUL;
+							/* libwrap sets it to unknown if no identreply. */
+							if (strcmp((char *)state->auth.mdata.rfc931.name,
+							STRING_UNKNOWN) == 0)
+								*state->auth.mdata.rfc931.name = NUL;
 
-				if (state->auth.mdata.rfc931.name[
-				sizeof(state->auth.mdata.rfc931.name) - 1] != NUL) {
-					slog(LOG_INFO, "%s: rfc931 name truncated", function);
-					state->auth.mdata.rfc931.name[
-					sizeof(state->auth.mdata.rfc931.name) - 1] = NUL;
-				}
+							if (state->auth.mdata.rfc931.name[
+							sizeof(state->auth.mdata.rfc931.name) - 1] != NUL) {
+								slog(LOG_NOTICE, "%s: rfc931 name truncated", function);
+								state->auth.mdata.rfc931.name[
+								sizeof(state->auth.mdata.rfc931.name) - 1] = NUL;
 
-				if (*state->auth.mdata.rfc931.name != NUL)
-					state->auth.method = AUTHMETHOD_RFC931;
-				else
-					continue;
-			}
-			else
+								/* better safe than sorry. */
+								*state->auth.mdata.rfc931.name = NUL;
+							}
+
+							if (*state->auth.mdata.rfc931.name != NUL)
+								methodisok = 1;
+							break;
 #endif /* HAVE_LIBWRAP */
-				continue;
+
+#if HAVE_PAM
+						case AUTHMETHOD_PAM:
+							slog(LOG_DEBUG, "%s: trying to find match for pam ...",
+							function);
+
+							switch (state->auth.method) {
+								case AUTHMETHOD_UNAME: {
+									/* it's a union, make a copy first. */
+									const struct authmethod_uname_t uname
+									= state->auth.mdata.uname;
+
+									/* similar enough, just copy name/password. */
+
+									strcpy((char *)state->auth.mdata.pam.name, 
+									(const char *)uname.name);
+									strcpy((char *)state->auth.mdata.pam.password, 
+									(const char *)uname.password);
+
+									methodisok = 1;
+									break;
+								}
+
+								case AUTHMETHOD_RFC931: {
+									/* it's a union, make a copy first. */
+									const struct authmethod_rfc931_t rfc931
+									= state->auth.mdata.rfc931;
+
+									strcpy((char *)state->auth.mdata.pam.name, 
+									(const char *)rfc931.name);
+									*state->auth.mdata.pam.password = NUL;
+									methodisok = 1;
+									break;
+								}
+
+								case AUTHMETHOD_NONE:
+									*state->auth.mdata.pam.name 		= NUL;
+									*state->auth.mdata.pam.password	= NUL;
+									methodisok = 1;
+									break;
+
+							}
+
+							strcpy(state->auth.mdata.pam.servicename,
+							rule->pamservicename);
+#endif /* HAVE_PAM */
+					}
+
+					if (methodisok) {
+						state->auth.method = methodv[i];
+						break;
+					}
+				}
+			}
+
+			if (i == methodc)
+				continue;	/* no usable method found. */
+		}
+
 
 		SASSERTX(methodisset(state->auth.method, rule->state.methodv,
-		(size_t)rule->state.methodc));
+		rule->state.methodc));
 
-		switch (state->auth.method) {
-			case AUTHMETHOD_UNAME:
-				name		= (char *)state->auth.mdata.uname.name;
-				password	= (char *)state->auth.mdata.uname.password;
-				break;
+		i = accessmatch(s, &state->auth, peer, local, rule->user, msg, msgsize);
 
-			case AUTHMETHOD_RFC931:
-				name		= (char *)state->auth.mdata.rfc931.name;
-				password	= NULL;
-				break;
-
-			default:
-				/* not usernamebased. */
-				name = password = NULL;
-		}
-
-		if (name != NULL && rule->user != NULL) {
-			struct linkedname_t *ruleuser;
+		/* two fields we want to copy. */
+		memcpy(ostate.auth.methodv, state->auth.methodv,
+		state->auth.methodc * sizeof(*state->auth.methodv));
+		ostate.auth.methodc = state->auth.methodc;
+		memcpy(ostate.auth.badmethodv, state->auth.badmethodv,
+		state->auth.badmethodc * sizeof(*state->auth.badmethodv));
+		ostate.auth.badmethodc = state->auth.badmethodc;
 
+		if (!i)
 			/*
-			 * The rule->user names restricts access further, only names
-			 * appearing there are checked.
+			 * The reason for the continue is the fact that we can
+			 * "upgrade" the method if we have a rule specifing a 
+			 * non-socks method.  That means "name" and "password" 
+			 * gotten for this method/rule need not be the same as gotten
+			 * for other methods.
 			 */
-			ruleuser = rule->user;
-			do
-				if (strcmp(name, ruleuser->name) == 0)
-					break;
-			while ((ruleuser = ruleuser->next) != NULL);
-
-			if (ruleuser == NULL)
-				continue; /* no match. */
-		}
-
-		if (name != NULL)
-			if (!state->auth.matched) { /* may have been checked at lower level. */
-				/* name must also be in passwordfile. */
-				if (passwordcheck(name, password) != 0)
-					continue;
-				state->auth.matched = 1;
-			}
+			continue; 
 
 		/*
-		 * This is a little tricky, but for some commands we may not
+		 * This is a little tricky.  For some commands we may not
 		 * have all info at time of (preliminary) rulechecks.
 		 * What we want to do if there is no (complete) address given is
 		 * to see if there's any chance at all the rules will permit this
@@ -890,6 +967,7 @@ rulespermit(s, match, state, src, dst)
 
 	if (rule == NULL) /* no rules matched; match default rule. */
 		rule = &defrule;
+
 	*match = *rule;
 
 	/*
@@ -943,6 +1021,10 @@ authinfo(auth, info, infolen)
 				authname = (const char *)auth->mdata.rfc931.name;
 				break;
 
+			case AUTHMETHOD_PAM:
+				authname = (const char *)auth->mdata.pam.name;
+				break;
+
 			default:
 				SERRX(auth->method);
 		}
@@ -958,6 +1040,14 @@ authinfo(auth, info, infolen)
 	return info;
 }
 
+const char *
+verdict2string(verdict)
+	int verdict;
+{
+
+	return verdict == VERDICT_PASS ? VERDICT_PASSs : VERDICT_BLOCKs;
+}
+
 
 static struct rule_t *
 addrule(newrule, rulebase, client)
@@ -968,7 +1058,19 @@ addrule(newrule, rulebase, client)
 	static const struct serverstate_t state;
 	const char *function = "addrule()";
 	struct rule_t *rule;
-	int i;
+	size_t i;
+	int *methodv;
+	size_t methodc;
+
+	if (client) {
+		methodv = config.clientmethodv;
+		methodc = config.clientmethodc;
+	}
+	else {
+		methodv = config.methodv;
+		methodc = config.methodc;
+	}
+
 
 	if ((rule = malloc(sizeof(*rule))) == NULL)
 		serrx(EXIT_FAILURE, "%s: %s", function, NOMEM);
@@ -989,59 +1091,25 @@ addrule(newrule, rulebase, client)
 		memset(&rule->state.command, UCHAR_MAX, sizeof(rule->state.command));
 
 	/*
-	 * If no method set, set all set in the global config.methodv.
+	 * If no method set, set all set from global methodline that make sense.
 	 */
 	if (rule->state.methodc == 0) {
-		int *methodv = rule->state.methodv;
-		int *methodc = &rule->state.methodc;
-
-		for (i = 0; i < config.methodc; ++i) {
-			int addmethod = 0;
-
-			if (client) {
-				switch (config.methodv[i]) {
-					case AUTHMETHOD_NONE:
-						if (rule->user != NULL)
-							break; /* can't get username with this method. */
-						addmethod = 1;
+		for (i = 0; i < methodc; ++i)
+			switch (methodv[i]) {
+				case AUTHMETHOD_NONE:
+					if (rule->user != NULL)
 						break;
+					/* else; */ /* FALLTHROUGH */
 
-					case AUTHMETHOD_RFC931:
-						addmethod = 1;
-						break;
-
-					default:  /* some non-client method presumably. */
-						break;
-				}
+				default:
+					rule->state.methodv[rule->state.methodc++] = methodv[i];
 			}
-			else {
-				switch (config.methodv[i]) {
-					case AUTHMETHOD_NONE:
-						if (rule->user != NULL)
-							break; /* can't get username with this method. */
-						addmethod = 1;
-						break;
-
-					case AUTHMETHOD_UNAME:
-					case AUTHMETHOD_RFC931:
-						addmethod = 1;
-						break;
-
-					default:
-						SERRX(config.methodv[i]);
-				}
-			}
-
-			if (addmethod)
-				methodv[(*methodc)++] = config.methodv[i];
-		}
 	}
 
 	/* warn about methods not set in the global method?  May not be an error. */
 	for (i = 0; i < rule->state.methodc; ++i)
-		if (!methodisset(rule->state.methodv[i], (const int *)&config.methodv,
-		(size_t)config.methodc))
-			yywarn("method \"%s\" set in rule but not in global method",
+		if (!methodisset(rule->state.methodv[i], methodv, methodc))
+			yywarn("method \"%s\" set in rule but not in global methodline",
 			method2string(rule->state.methodv[i]));
 
 	/* if no protocol set, set all. */
@@ -1098,9 +1166,35 @@ static void
 checkrule(rule)
 	const struct rule_t *rule;
 {
+	size_t i;
 
 	if (rule->state.methodc == 0)
 		yywarn("rule allows no methods");
+
+	if (rule->user != NULL) {
+		/* check that all methods given provide usernames. */
+		for (i = 0; i < rule->state.methodc; ++i)
+			switch (rule->state.methodv[i]) {
+				case AUTHMETHOD_UNAME:
+				case AUTHMETHOD_RFC931:
+				case AUTHMETHOD_PAM:
+					break;
+
+				default:
+					yyerror("method \"%s\" can not provide usernames",
+					method2string(rule->state.methodv[i]));
+			}
+	}
+
+#if HAVE_PAM 
+	if (*rule->pamservicename != NUL)
+		if (!methodisset(AUTHMETHOD_PAM, rule->state.methodv,
+		rule->state.methodc))
+			yyerror("pamservicename set for rule but not method pam");
+		else 
+			if (strcmp(rule->pamservicename, DEFAULT_PAMSERVICENAME) != 0)
+				config.state.unfixedpamdata = 1; /* pamservicename varies. */
+#endif /* HAVE_PAM */
 }
 
 static void
@@ -1223,3 +1317,4 @@ connectisok(request, rule)
 
 	return 1;
 }
+
