@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: sockd.c,v 1.268 2001/02/20 18:27:04 karls Exp $";
+"$Id: sockd.c,v 1.272 2001/05/11 09:14:42 michaels Exp $";
 
 	/*
 	 * signal handlers
@@ -114,6 +114,15 @@ char *__progname = "sockd";	/* default. */
 
 extern char *optarg;
 
+#define ELECTRICFENCE	0
+
+#if ELECTRICFENCE
+	extern int EF_PROTECT_FREE;
+	extern int EF_ALLOW_MALLOC_0;
+	extern int EF_ALIGNMENT;
+	extern int EF_PROTECT_BELOW;
+#endif /* ELECTRICFENCE */
+
 __END_DECLS
 
 int
@@ -152,6 +161,12 @@ main(argc, argv, envp)
 	malloc_options = "AJ";
 #endif  /* DIAGNOSTIC && HAVE_MALLOC_OPTIONS */
 
+#if ELECTRICFENCE
+	EF_PROTECT_FREE         = 1;
+	EF_ALLOW_MALLOC_0       = 1;
+	EF_ALIGNMENT            = 0;
+	EF_PROTECT_BELOW			= 0;
+#endif /* ELECTRICFENCE */
 
 	serverinit(argc, argv, envp);
 	showconfig(&config);
@@ -587,19 +602,6 @@ main(argc, argv, envp)
 	/* NOTREACHED */
 }
 
-int
-pidismother(pid)
-	pid_t pid;
-{
-	int i;
-
-	for (i = 0; i < config.option.serverc; ++i)
-		if (config.state.motherpidv[i] == pid)
-			return i + 1;
-	return 0;
-}
-
-
 static void
 usage(code)
 	int code;
@@ -798,15 +800,14 @@ serverinit(argc, argv, envp)
 		config.option.configfile = SOCKD_CONFIGFILE;
 
 	genericinit();
+
 	checksettings();
 
-
 	if (verifyonly) {
-			  showconfig(&config);
-			  exit(EXIT_SUCCESS);
+		showconfig(&config);
+		exit(EXIT_SUCCESS);
 	}
 
-	socks_seteuid(&euid, config.uid.privileged);
 	for (i = 0; i < config.internalc; ++i) {
 		int flags;
 		struct listenaddress_t *l = &config.internalv[i];
@@ -821,6 +822,7 @@ serverinit(argc, argv, envp)
 		!= 0)
 			swarn("%s: setsockopt(SO_REUSEADDR)", function);
 
+		socks_seteuid(&euid, config.uid.privileged);
 		/* LINTED pointer casts may be troublesome */
 		if (sockd_bind(l->s, (struct sockaddr *)&l->addr, 0) != 0) {
 			char badbind[MAXSOCKADDRSTRING];
@@ -830,6 +832,7 @@ serverinit(argc, argv, envp)
 			function, sockaddr2string((struct sockaddr *)&l->addr, badbind,
 			sizeof(badbind)));
 		}
+		socks_reseteuid(config.uid.privileged, euid);
 
 		if (listen(l->s, SOCKD_MAXCLIENTQUE) == -1)
 			serr(EXIT_FAILURE, "%s: listen(%d)", function, SOCKD_MAXCLIENTQUE);
@@ -844,7 +847,6 @@ serverinit(argc, argv, envp)
 				serr(EXIT_FAILURE, "%s: socks_mklock()", function);
 #endif
 	}
-	socks_reseteuid(config.uid.privileged, euid);
 }
 
 static void
@@ -915,6 +917,9 @@ checksettings(void)
 	if (config.methodc == 0)
 		swarnx("%s: no methods enabled (total block)", function);
 
+	if (config.clientmethodc == 0)
+		config.clientmethodv[config.clientmethodc++] = AUTHMETHOD_NONE;
+		
 	if (!config.uid.privileged_isset)
 		config.uid.privileged = config.state.euid;
 	else {
