@@ -45,7 +45,7 @@
 #include "config_parse.h"
 
 static const char rcsid[] =
-"$Id: sockd_io.c,v 1.182 2001/02/06 15:59:09 michaels Exp $";
+"$Id: sockd_io.c,v 1.184 2001/05/02 10:57:16 michaels Exp $";
 
 /*
  * Accept io objects from mother and does io on them.  We never
@@ -452,8 +452,7 @@ delete_io(mother, io, fd, status)
 
 		authinfo(&io->src.auth, in, sizeof(in)); p = strlen(in);
 		/* LINTED pointer casts may be troublesome */
-		sockaddr2string((struct sockaddr *)&io->src.raddr, &in[p],
-		sizeof(in) - p);
+		sockaddr2string(&io->src.raddr, &in[p], sizeof(in) - p);
 
 		authinfo(&io->dst.auth, out, sizeof(out));
 		p = strlen(out);
@@ -463,8 +462,7 @@ delete_io(mother, io, fd, status)
 			case SOCKS_BINDREPLY:
 			case SOCKS_CONNECT:
 				/* LINTED pointer casts may be troublesome */
-				sockaddr2string((struct sockaddr *)&io->dst.raddr, &out[p],
-				sizeof(out) - p);
+				sockaddr2string(&io->dst.raddr, &out[p], sizeof(out) - p);
 				break;
 
 			case SOCKS_UDPASSOCIATE:
@@ -637,6 +635,7 @@ recv_io(s, io)
 	msg.msg_name			= NULL;
 	msg.msg_namelen		= 0;
 
+	/* LINTED pointer casts may be troublesome */
 	CMSG_SETHDR_RECV(sizeof(cmsgmem));
 
 	if (recvmsgn(s, &msg, 0, length) != (ssize_t)length) {
@@ -678,13 +677,16 @@ recv_io(s, io)
 
 	fdreceived = 0;
 
+	/* LINTED pointer casts may be troublesome */
 	CMSG_GETOBJECT(io->src.s, sizeof(io->src.s) * fdreceived++);
+	/* LINTED pointer casts may be troublesome */
 	CMSG_GETOBJECT(io->dst.s, sizeof(io->dst.s) * fdreceived++);
 
 	switch (io->state.command) {
 		case SOCKS_BIND:
 		case SOCKS_BINDREPLY:
 			if (io->state.extension.bind)
+				/* LINTED pointer casts may be troublesome */
 				CMSG_GETOBJECT(io->control.s, sizeof(io->control.s) * fdreceived++);
 			else
 				io->control.s = -1;
@@ -695,6 +697,7 @@ recv_io(s, io)
 			break;
 
 		case SOCKS_UDPASSOCIATE:
+			/* LINTED pointer casts may be troublesome */
 			CMSG_GETOBJECT(io->control.s, sizeof(io->control.s) * fdreceived++);
 			break;
 
@@ -838,21 +841,22 @@ doio(mother, io, rset, wset, flags)
 				 * assume the first packet is from is it.
 				 * Client can only blame itself if not.
 				 */
-				if (io->src.raddr.sin_addr.s_addr	== htonl(INADDR_ANY)
-				||  io->src.raddr.sin_port				== htons(0)) {
-					if (io->src.raddr.sin_addr.s_addr == htonl(INADDR_ANY))
-					/* LINTED pointer casts may be troublesome */
-						io->src.raddr.sin_addr.s_addr
-						= ((struct sockaddr_in *)&from)->sin_addr.s_addr;
 
-					if (io->src.raddr.sin_port == htons(0))
+				/* LINTED pointer casts may be troublesome */
+				if (TOIN(&io->src.raddr)->sin_addr.s_addr == htonl(INADDR_ANY)
+				||  TOIN(&io->src.raddr)->sin_port == htons(0)) {
+					/* LINTED pointer casts may be troublesome */
+					if (TOIN(&io->src.raddr)->sin_addr.s_addr == htonl(INADDR_ANY))
+					/* LINTED pointer casts may be troublesome */
+						TOIN(&io->src.raddr)->sin_addr.s_addr
+						= TOIN(&from)->sin_addr.s_addr;
+
+					/* LINTED pointer casts may be troublesome */
+					if (TOIN(&io->src.raddr)->sin_port == htons(0))
 						/* LINTED pointer casts may be troublesome */
-						io->src.raddr.sin_port
-						= ((struct sockaddr_in *)&from)->sin_port;
+						TOIN(&io->src.raddr)->sin_port = TOIN(&from)->sin_port;
 
-					/* LINTED pointer casts may be troublesome */
-					sockaddr2sockshost((struct sockaddr *)&io->src.raddr,
-					&io->src.host);
+					sockaddr2sockshost(&io->src.raddr, &io->src.host);
 				}
 
 				/*
@@ -863,17 +867,15 @@ doio(mother, io, rset, wset, flags)
 				if (io->src.read == 0) { /* could happen more than once, but ok. */
 					struct connectionstate_t rstate;
 
-					/* LINTED pointer casts may be troublesome */
-					if (!sockaddrareeq((struct sockaddr *)&io->src.raddr, &from)) {
+					if (!sockaddrareeq(&io->src.raddr, &from)) {
 						char src[MAXSOCKADDRSTRING], dst[MAXSOCKADDRSTRING];
 
 						/* perhaps this should be LOG_DEBUG. */
 						slog(LOG_NOTICE,
 						"%s(0): %s: expected from %s, got it from %s",
 						VERDICT_BLOCKs, protocol2string(io->state.protocol),
-						/* LINTED pointer casts may be troublesome */
-						sockaddr2string((struct sockaddr *)&io->src.raddr, src,
-						sizeof(src)), sockaddr2string(&from, dst, sizeof(dst)));
+						sockaddr2string(&io->src.raddr, src, sizeof(src)),
+						sockaddr2string(&from, dst, sizeof(dst)));
 						break;
 					}
 
@@ -885,10 +887,12 @@ doio(mother, io, rset, wset, flags)
 					rstate			= io->state;
 					rstate.command	= SOCKS_UDPREPLY;
 
-					if (!rulespermit(io->src.s, &io->rule, &io->state, &io->src.host,
-					NULL)
-					&&  !rulespermit(io->src.s, &io->rule, &rstate, NULL,
-					&io->src.host)) {
+					if (!rulespermit(io->src.s, &io->control.raddr,
+					&io->control.laddr, &io->rule, &io->state, &io->src.host,
+					NULL, NULL, 0)
+					&&  !rulespermit(io->src.s, &io->control.raddr,
+					&io->control.laddr, &io->rule, &rstate, NULL, &io->src.host,
+					NULL, 0)) {
 						/* can't send, can't receive; drop it. */
 						delete_io(mother, io, io->src.s, IO_SRCBLOCK);
 						return;
@@ -902,8 +906,8 @@ doio(mother, io, rset, wset, flags)
 
 					/* LINTED pointer casts may be troublesome */
 					swarnx("%s: bad socks udppacket (length = %d) from %s",
-					function, r, sockaddr2string((struct sockaddr *)&io->src.raddr,
-					badfrom, sizeof(badfrom)));
+					function, r, sockaddr2string(&io->src.raddr, badfrom,
+					sizeof(badfrom)));
 					break;
 				}
 
@@ -913,16 +917,16 @@ doio(mother, io, rset, wset, flags)
 					/* LINTED pointer casts may be troublesome */
 					swarnx("%s: %s: fragmented packet from %s.  Not supported",
 					function, protocol2string(io->state.protocol),
-					sockaddr2string((struct sockaddr *)&io->src.raddr, badfrom,
-					sizeof(badfrom)));
+					sockaddr2string(&io->src.raddr, badfrom, sizeof(badfrom)));
 					break;
 				}
 
 				io->dst.host = header.host;
 
 				/* is the packet to be permitted out? */
-				permit = rulespermit(io->src.s, &io->rule, &io->state,
-				&io->src.host, &io->dst.host);
+				permit = rulespermit(io->src.s, &io->control.raddr,
+				&io->control.laddr, &io->rule, &io->state, &io->src.host,
+				&io->dst.host, NULL, 0);
 
 				/* set r to bytes sent by client sans socks UDP header. */
 				r -= PACKETSIZE_UDP(&header);
@@ -935,12 +939,12 @@ doio(mother, io, rset, wset, flags)
 					break;
 
 				/* LINTED pointer casts may be troublesome */
-				sockshost2sockaddr(&header.host, (struct sockaddr *)&io->dst.raddr);
+				sockshost2sockaddr(&header.host, &io->dst.raddr);
 
 				/* LINTED pointer casts may be troublesome */
 				if ((w = socks_sendto(io->dst.s, &buf[PACKETSIZE_UDP(&header)],
-				(size_t)r, lflags, (struct sockaddr *)&io->dst.raddr,
-				sizeof(io->dst.raddr), &io->dst.auth)) != r)
+				(size_t)r, lflags, &io->dst.raddr, sizeof(io->dst.raddr),
+				&io->dst.auth)) != r)
 					iolog(&io->rule, &io->state, OPERATION_ERROR, &io->src.host,
 					&io->src.auth, &io->dst.host, &io->dst.auth, NULL, 0);
 				io->dst.written += MAX(0, w);
@@ -989,7 +993,7 @@ doio(mother, io, rset, wset, flags)
 
 				/* LINTED possible pointer alignment problem */
 				if (io->dst.host.atype == SOCKS_ADDR_DOMAIN
-				&& sockaddrareeq((struct sockaddr *)&io->dst.raddr, &from))
+				&& sockaddrareeq(&io->dst.raddr, &from))
 					srcsh = io->dst.host;
 				else
 					sockaddr2sockshost(&from, &srcsh);
@@ -998,8 +1002,9 @@ doio(mother, io, rset, wset, flags)
 				state				= io->state;
 				state.command	= SOCKS_UDPREPLY;
 
-				permit
-				= rulespermit(io->dst.s, &io->rule, &state, &srcsh, &io->src.host);
+				permit = rulespermit(io->dst.s, &io->control.raddr,
+				&io->control.laddr, &io->rule, &state, &srcsh, &io->src.host,
+				NULL, 0);
 
 				io->dst.auth = io->state.auth;
 
@@ -1054,8 +1059,8 @@ doio(mother, io, rset, wset, flags)
 
 			slog(LOG_NOTICE, "%s/control: %d unexpected bytes: %s",
 			/* LINTED pointer casts may be troublesome */
-			sockaddr2string((struct sockaddr *)&io->control.raddr, hmmread,
-			sizeof(hmmread)), r, strcheck(unexpected = str2vis(buf, r)));
+			sockaddr2string(&io->control.raddr, hmmread, sizeof(hmmread)), r,
+			strcheck(unexpected = str2vis(buf, r)));
 
 			free(unexpected);
 		}
