@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: sockd_protocol.c,v 1.88 2001/02/06 15:59:11 michaels Exp $";
+"$Id: sockd_protocol.c,v 1.90 2001/05/08 08:47:31 michaels Exp $";
 
 __BEGIN_DECLS
 
@@ -212,6 +212,7 @@ recv_v5req (s, request, state)
 	/* NMETHODS */
 	INIT(sizeof(char));
 	CHECK(&state->mem[start], request->auth, NULL);
+	/* LINTED conversion from 'int' may lose accuracy */
 	OCTETIFY(state->mem[start]);
 
 	state->rcurrent = recv_methods;
@@ -235,7 +236,8 @@ recv_methods(s, request, state)
 	INIT(methodc);
 	CHECK(&state->mem[start], request->auth, NULL);
 
-	request->auth->method = selectmethod(&state->mem[start], (size_t)methodc);
+	request->auth->method = selectmethod(config.methodv, config.methodc,
+	&state->mem[start], (size_t)methodc);
 
 	/* send reply:
 	 *
@@ -255,8 +257,11 @@ recv_methods(s, request, state)
 	if (writen(s, reply, sizeof(reply), request->auth) != sizeof(reply))
 		return -1;
 
-	if (request->auth->method == AUTHMETHOD_NOACCEPT)
+	if (request->auth->method == AUTHMETHOD_NOACCEPT) {
+		snprintf(state->emsg, sizeof(state->emsg),
+		"client offered no acceptable authenticationmethod");
 		return -1;
+	}
 
 	state->rcurrent = methodnegotiate;
 	return state->rcurrent(s, request, state);
@@ -422,6 +427,7 @@ recv_address(s, request, state)
 					INIT(sizeof(*request->host.addr.domain));
 					CHECK(request->host.addr.domain, request->auth, NULL);
 
+					/* LINTED conversion from 'int' may lose accuracy */
 					OCTETIFY(*request->host.addr.domain);
 
 					state->rcurrent = recv_domain;
