@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: sockd_request.c,v 1.126 2001/02/06 15:59:12 michaels Exp $";
+"$Id: sockd_request.c,v 1.130 2001/05/13 14:26:50 michaels Exp $";
 
 /*
  * Since it only handles one client at a time there is no possibility
@@ -137,8 +137,7 @@ run_request(mother)
 		if (recv_req(mother->s, &req) == -1)
 			sockdexit(-EXIT_FAILURE);
 
-		/* LINTED pointer casts may be troublesome */
-		proctitleupdate((struct sockaddr *)&req.from);
+		proctitleupdate(&req.from);
 
 		dorequest(mother->s, &req);
 
@@ -216,10 +215,11 @@ dorequest(mother, request)
 {
 	const char *function = "dorequest()";
 	static const struct sockd_io_t ioinit;
-	struct sockaddr_in bound;
+	struct sockaddr bound;
 	struct sockd_io_t io;
 	struct response_t response;
 	char a[MAXSOCKSHOSTSTRING], b[MAXSOCKSHOSTSTRING];
+	char msg[256];
 	int p, permit, out;
 
 	slog(LOG_DEBUG, "received request: %s",
@@ -251,10 +251,9 @@ dorequest(mother, request)
 					break;
 
 				default:
-					/* LINTED pointer casts may be troublesome */
 					slog(LOG_INFO, "%s: unrecognized v%d command: %d",
-					sockaddr2string((const struct sockaddr *)&request->from,
-					a, sizeof(a)), request->req.version, request->req.command);
+					sockaddr2string(&request->from, a, sizeof(a)),
+					request->req.version, request->req.command);
 					send_failure(request->s, &response, SOCKS_FAILURE);
 					close(request->s);
 					return;
@@ -268,8 +267,8 @@ dorequest(mother, request)
 				default:
 					/* LINTED pointer casts may be troublesome */
 					slog(LOG_INFO, "%s: unrecognized v%d address type: %d",
-					sockaddr2string((const struct sockaddr *)&request->from,
-					a, sizeof(a)), request->req.version, request->req.host.atype);
+					sockaddr2string(&request->from, a, sizeof(a)),
+					request->req.version, request->req.host.atype);
 					send_failure(request->s, &response, SOCKS_ADDR_UNSUPP);
 					close(request->s);
 					return;
@@ -293,8 +292,8 @@ dorequest(mother, request)
 				default:
 					/* LINTED pointer casts may be troublesome */
 					slog(LOG_INFO, "%s: unrecognized v%d command: %d",
-					sockaddr2string((const struct sockaddr *)&request->from,
-					a, sizeof(a)), request->req.version, request->req.command);
+					sockaddr2string(&request->from, a, sizeof(a)),
+					request->req.version, request->req.command);
 					send_failure(request->s, &response, SOCKS_CMD_UNSUPP);
 					close(request->s);
 					return;
@@ -309,8 +308,8 @@ dorequest(mother, request)
 				default:
 					/* LINTED pointer casts may be troublesome */
 					slog(LOG_INFO, "%s: unrecognized v%d address type: %d",
-					sockaddr2string((const struct sockaddr *)&request->from,
-					a, sizeof(a)), request->req.version, request->req.host.atype);
+					sockaddr2string(&request->from, a, sizeof(a)),
+					request->req.version, request->req.host.atype);
 					send_failure(request->s, &response, SOCKS_ADDR_UNSUPP);
 					close(request->s);
 					return;
@@ -327,7 +326,7 @@ dorequest(mother, request)
 	 */
 
 	/* LINTED pointer casts may be troublesome */
-	sockaddr2sockshost((const struct sockaddr *)&request->from,
+	sockaddr2sockshost(&request->from,
 	&io.control.host);
 
 	switch (request->req.command) {
@@ -392,8 +391,7 @@ dorequest(mother, request)
 	/* find address to bind on clients behalf. */
 	switch ((*config.externalv).atype) {
 		case SOCKS_ADDR_IFNAME:
-			if (ifname2sockaddr((*config.externalv).addr.ifname,
-			(struct sockaddr *)&bound) == NULL) {
+			if (ifname2sockaddr((*config.externalv).addr.ifname, &bound) == NULL) {
 				slog(LOG_ALERT, "%s: can't find external interface/address: %s",
 				function, (*config.externalv).addr.ifname);
 				close(request->s);
@@ -405,7 +403,7 @@ dorequest(mother, request)
 			struct sockshost_t host;
 
 			sockshost2sockaddr(ruleaddress2sockshost(&*config.externalv, &host,
-			SOCKS_TCP), (struct sockaddr *)&bound);
+			SOCKS_TCP), &bound);
 			break;
 		}
 
@@ -419,14 +417,17 @@ dorequest(mother, request)
 			switch (request->req.version) {
 				case SOCKS_V4:
 					if (io.state.extension.bind)
-						bound.sin_port	= io.dst.host.port;
+						/* LINTED pointer casts may be troublesome */
+						TOIN(&bound)->sin_port = io.dst.host.port;
 					else
 						/* best we can try for is to use same port as source. */
-						bound.sin_port	= request->from.sin_port;
+						/* LINTED pointer casts may be troublesome */
+						TOIN(&bound)->sin_port = TOCIN(&request->from)->sin_port;
 					break;
 
 				case SOCKS_V5:
-					bound.sin_port = io.dst.host.port;
+					/* LINTED pointer casts may be troublesome */
+					TOIN(&bound)->sin_port = io.dst.host.port;
 					break;
 
 				default:
@@ -435,11 +436,13 @@ dorequest(mother, request)
 			break;
 
 		case SOCKS_CONNECT:
-			bound.sin_port	= request->from.sin_port;
+			/* LINTED pointer casts may be troublesome */
+			TOIN(&bound)->sin_port = TOCIN(&request->from)->sin_port;
 			break;
 
 		case SOCKS_UDPASSOCIATE:
-			bound.sin_port	= request->req.host.port;
+			/* LINTED pointer casts may be troublesome */
+			TOIN(&bound)->sin_port	= request->req.host.port;
 			break;
 
 		default:
@@ -457,24 +460,23 @@ dorequest(mother, request)
 			swarn("%s: setsockopt(SO_REUSEADDR)", function);
 	}
 
-	if (PORTISRESERVED(bound.sin_port) && config.compat.sameport) {
+	/* LINTED pointer casts may be troublesome */
+	if (PORTISRESERVED(TOIN(&bound)->sin_port) && config.compat.sameport) {
 		uid_t euid;
 
 		socks_seteuid(&euid, config.uid.privileged);
-		p = bindresvport(out, &bound);
+		/* LINTED pointer casts may be troublesome */
+		p = bindresvport(out, TOIN(&bound));
 		socks_reseteuid(config.uid.privileged, euid);
 	}
 	else
-		/* LINTED pointer casts may be troublesome */
-		p = sockd_bind(out, (struct sockaddr *)&bound, 1);
+		p = sockd_bind(out, &bound, 1);
 
 	if (p != 0) { /* no such luck, bind any port and let client decide if ok. */
-		bound.sin_port	= htons(0);
 		/* LINTED pointer casts may be troublesome */
-		if ((p = sockd_bind(out, (struct sockaddr *)&bound, 0)) != 0)
-			swarn("%s: bind(%s)",
-			/* LINTED pointer casts may be troublesome */
-			function, sockaddr2string((struct sockaddr *)&bound, a, sizeof(a)));
+		TOIN(&bound)->sin_port = htons(0);
+		if ((p = sockd_bind(out, &bound, 0)) != 0)
+			swarn("%s: bind(%s)", function, sockaddr2string(&bound, a, sizeof(a)));
 	}
 
 	if (p != 0) {
@@ -493,33 +495,33 @@ dorequest(mother, request)
 
 			boundlen = sizeof(bound);
 			/* LINTED pointer casts may be troublesome */
-			if (getsockname(out, (struct sockaddr *)&bound, &boundlen) != 0) {
+			if (getsockname(out, &bound, &boundlen) != 0) {
 				swarn("%s: getsockname(out)", function);
 				close(request->s);
 				close(out);
 				return;
 			}
 			/* LINTED pointer casts may be troublesome */
-			sockaddr2sockshost((struct sockaddr *)&bound, &boundhost);
+			sockaddr2sockshost(&bound, &boundhost);
 
-			permit = rulespermit(request->s, &io.rule, &io.state, &io.src.host,
-			&boundhost);
+			permit = rulespermit(request->s, &request->from, &request->to, 
+			&io.rule, &io.state, &io.src.host, &boundhost, msg, sizeof(msg));
 
 			io.src.auth = io.control.auth = io.state.auth;
 
 			iolog(&io.rule, &io.state, OPERATION_CONNECT, &io.src.host,
-			&io.src.auth, &boundhost, &io.dst.auth, NULL, 0);
+			&io.src.auth, &boundhost, &io.dst.auth, msg, strlen(msg));
 			break;
 		}
 
 		case SOCKS_CONNECT:
-			permit = rulespermit(request->s, &io.rule, &io.state, &io.src.host,
-			&io.dst.host);
+			permit = rulespermit(request->s, &request->from, &request->to, 
+			&io.rule, &io.state, &io.src.host, &io.dst.host, msg, sizeof(msg));
 
 			io.src.auth = io.control.auth = io.state.auth;
 
 			iolog(&io.rule, &io.state, OPERATION_CONNECT, &io.src.host,
-			&io.src.auth, &io.dst.host, &io.dst.auth, NULL, 0);
+			&io.src.auth, &io.dst.host, &io.dst.auth, msg, strlen(msg));
 			break;
 
 		case SOCKS_UDPASSOCIATE: {
@@ -541,13 +543,15 @@ dorequest(mother, request)
 			replystate.command	= SOCKS_UDPREPLY;
 
 			/* one direction is atleast in theory good enough. */
-			permit = rulespermit(request->s, &io.rule, &io.state, src, NULL)
-					|| rulespermit(request->s, &io.rule, &replystate, NULL, src);
+			permit = rulespermit(request->s, &request->from, &request->to,
+			&io.rule, &io.state, src, NULL, msg, sizeof(msg))
+			|| rulespermit(request->s, &request->from, &request->to, 
+			&io.rule, &replystate, NULL, src, msg, sizeof(msg));
 
 			io.src.auth = io.control.auth = io.state.auth;
 
 			iolog(&io.rule, &io.state, OPERATION_CONNECT, &io.src.host,
-			&io.src.auth, &io.dst.host, &io.dst.auth, NULL, 0);
+			&io.src.auth, &io.dst.host, &io.dst.auth, msg, strlen(msg));
 			break;
 		}
 
@@ -610,8 +614,7 @@ dorequest(mother, request)
 			sockaddr2sockshost(&boundaddr, &response.host);
 			response.reply	= (char)sockscode(response.version, SOCKS_SUCCESS);
 
-			/* LINTED pointer casts may be troublesome */
-			clientaddr = *(const struct sockaddr *)&request->from;
+			clientaddr = request->from;
 
 			if (io.state.extension.bind) {
 				int pipev[2];
@@ -639,7 +642,7 @@ dorequest(mother, request)
 				sv[ourpipe]		= pipev[1];
 
 				/* LINTED pointer casts may be troublesome */
-				((struct sockaddr_in *)&clientaddr)->sin_port = io.dst.host.port;
+				TOIN(&clientaddr)->sin_port = io.dst.host.port;
 			}
 
 			/* let client know what address we bound to on it's behalf. */
@@ -668,7 +671,7 @@ dorequest(mother, request)
 				struct ruleaddress_t ruleaddr;
 				struct sockaddr remoteaddr;		/* remote address we accepted.	*/
 				struct sockshost_t remotehost;	/* remote address, sockhost form.*/
-				struct sockaddr_in replyaddr;		/* address of bindreply socket.	*/
+				struct sockaddr replyaddr;			/* address of bindreply socket.	*/
 				int fdbits = -1;
 				fd_set rset;
 
@@ -742,12 +745,9 @@ dorequest(mother, request)
 							else {
 								SASSERTX(fio->state.command = SOCKS_BINDREPLY);
 
-								/* LINTED pointer casts may be troublesome */
-								SASSERTX(sockaddrareeq((struct sockaddr *)
-								&fio->dst.laddr, &queryaddr));
+								SASSERTX(sockaddrareeq(&fio->dst.laddr, &queryaddr));
 
-								/* LINTED pointer casts may be troublesome */
-								sockaddr2sockshost((struct sockaddr *)&fio->src.raddr,
+								sockaddr2sockshost(&fio->src.raddr,
 								&queryresponse.host);
 							}
 
@@ -820,14 +820,15 @@ dorequest(mother, request)
 
 					bindio.state.auth.method = AUTHMETHOD_NONE;
 
-					permit = rulespermit(sv[remote], &bindio.rule, &bindio.state,
-					&bindio.src.host, &bindio.dst.host);
+					permit = rulespermit(sv[remote], &request->from, &request->to,
+					&bindio.rule, &bindio.state, &bindio.src.host, &bindio.dst.host,
+					msg, sizeof(msg));
 
 					bindio.src.auth = bindio.state.auth;
 
 					iolog(&bindio.rule, &bindio.state, OPERATION_CONNECT,
 					&bindio.src.host, &bindio.src.auth, &bindio.dst.host,
-					&bindio.dst.auth, NULL, 0);
+					&bindio.dst.auth, msg, strlen(msg));
 
 				}
 				else {
@@ -871,23 +872,19 @@ dorequest(mother, request)
 					}
 					setsockoptions(sv[reply]);
 
+					replyaddr						= boundaddr;
 					/* LINTED pointer casts may be troublesome */
-					replyaddr				= *(struct sockaddr_in *)&boundaddr;
-					replyaddr.sin_port	= htons(0);
+					TOIN(&replyaddr)->sin_port	= htons(0);
 
-					/* LINTED pointer casts may be troublesome */
-					if (sockd_bind(sv[reply], (struct sockaddr *)&replyaddr, 0)
-					!= 0) {
-						/* LINTED pointer casts may be troublesome */
+					if (sockd_bind(sv[reply], &replyaddr, 0) != 0) {
 						swarn("%s: bind(%s)", function,
-						sockaddr2string((struct sockaddr *)&replyaddr, a, sizeof(a)));
+						sockaddr2string(&replyaddr, a, sizeof(a)));
 						break;
 					}
 
 					len = sizeof(replyaddr);
 					/* LINTED pointer casts may be troublesome */
-					if (getsockname(sv[reply], (struct sockaddr *)&replyaddr, &len)
-					!= 0) {
+					if (getsockname(sv[reply], &replyaddr, &len) != 0) {
 						swarn("%s: getsockname(sv[reply])", function);
 						if (errno == ENOBUFS) {
 							close(sv[remote]);
@@ -947,14 +944,11 @@ dorequest(mother, request)
 					bindio.dst			= bindio.control;
 					bindio.dst.laddr	= request->from;
 				}
-				/* LINTED pointer casts may be troublesome */
-				bindio.dst.raddr		= *(struct sockaddr_in *)&clientaddr;
+				bindio.dst.raddr		= clientaddr;
 
-				bindio.src.s			= sv[remote];
-				/* LINTED pointer casts may be troublesome */
-				bindio.src.laddr	= *(struct sockaddr_in *)&boundaddr;
-				/* LINTED pointer casts may be troublesome */
-				bindio.src.raddr	= *(struct sockaddr_in *)&remoteaddr;
+				bindio.src.s		= sv[remote];
+				bindio.src.laddr	= boundaddr;
+				bindio.src.raddr	= remoteaddr;
 
 				if (bindio.state.extension.bind)
 					/* add to list, client will query. */
@@ -989,8 +983,10 @@ dorequest(mother, request)
 			if (socks_connect(out, &io.dst.host) != 0) {
 				iolog(&io.rule, &io.state, OPERATION_ABORT, &io.src.host,
 				&io.src.auth, &io.dst.host, &io.dst.auth, NULL, 0);
+
 				send_failure(request->s, &response,
 				errno2reply(errno, response.version));
+
 				close(request->s);
 				break;
 			}
@@ -1003,9 +999,7 @@ dorequest(mother, request)
 
 			io.dst.s	= out;
 			sinlen	= sizeof(io.dst.raddr);
-			/* LINTED pointer casts may be troublesome */
-			if (getpeername(io.dst.s, (struct sockaddr *)&io.dst.raddr, &sinlen)
-			!= 0) {
+			if (getpeername(io.dst.s, &io.dst.raddr, &sinlen) != 0) {
 				swarn("%s: getpeername(io.dst.s)", function);
 				send_failure(request->s, &response, SOCKS_FAILURE);
 				close(request->s);
@@ -1013,17 +1007,14 @@ dorequest(mother, request)
 			}
 
 			sinlen = sizeof(io.dst.laddr);
-			/* LINTED pointer casts may be troublesome */
-			if (getsockname(io.dst.s, (struct sockaddr *)&io.dst.laddr, &sinlen)
-			!= 0) {
+			if (getsockname(io.dst.s, &io.dst.laddr, &sinlen) != 0) {
 				swarn("%s: getsockname(io.dst.s)", function);
 				send_failure(request->s, &response, SOCKS_FAILURE);
 				close(request->s);
 				break;
 			}
 
-			/* LINTED pointer casts may be troublesome */
-			sockaddr2sockshost((struct sockaddr *)&io.dst.laddr, &response.host);
+			sockaddr2sockshost(&io.dst.laddr, &response.host);
 			response.reply	= sockscode(response.version, SOCKS_SUCCESS);
 
 			flushio(mother, request->s, &response, &io);
@@ -1031,7 +1022,7 @@ dorequest(mother, request)
 		}
 
 		case SOCKS_UDPASSOCIATE: {
-			struct sockaddr_in client;
+			struct sockaddr client;
 			socklen_t boundlen;
 			int clientfd;
 
@@ -1044,27 +1035,25 @@ dorequest(mother, request)
 			}
 			setsockoptions(clientfd);
 
-			/* LINTED pointer casts may be troublesome */
-			sockshost2sockaddr(&request->req.host, (struct sockaddr *)&client);
+			sockshost2sockaddr(&request->req.host, &client);
 
 			io.control.s			= request->s;
 			io.control.laddr		= request->to;
 			io.control.raddr		= request->from;
 
-			io.src.s						= clientfd;
-			io.src.raddr				= client;
-			io.src.laddr				= request->to;
-			io.src.laddr.sin_port 	= htons(0);
+			io.src.s									= clientfd;
+			io.src.raddr							= client;
+			io.src.laddr							= request->to;
+			/* LINTED pointer casts may be troublesome */
+			TOIN(&io.src.laddr)->sin_port 	= htons(0);
 
 			/*
 			 * bind address for receiving UDP packets so we can tell client
 			 * where to send it's packets.
 			 */
-			/* LINTED pointer casts may be troublesome */
-			if (sockd_bind(clientfd, (struct sockaddr *)&io.src.laddr, 0) != 0) {
-				/* LINTED pointer casts may be troublesome */
+			if (sockd_bind(clientfd, &io.src.laddr, 0) != 0) {
 				swarn("%s: bind(%s)", function,
-				sockaddr2string((struct sockaddr *)&io.src.laddr, a, sizeof(a)));
+				sockaddr2string(&io.src.laddr, a, sizeof(a)));
 				send_failure(request->s, &response, SOCKS_FAILURE);
 				close(request->s);
 				close(clientfd);
@@ -1072,9 +1061,7 @@ dorequest(mother, request)
 			}
 
 			boundlen = sizeof(io.src.laddr);
-			/* LINTED pointer casts may be troublesome */
-			if (getsockname(clientfd, (struct sockaddr *)&io.src.laddr, &boundlen)
-			!= 0) {
+			if (getsockname(clientfd, &io.src.laddr, &boundlen) != 0) {
 				swarn("%s: getsockname(clientfd)", function);
 				send_failure(request->s, &response, SOCKS_FAILURE);
 				close(request->s);
@@ -1084,9 +1071,7 @@ dorequest(mother, request)
 
 			io.dst.s					= out;
 			boundlen = sizeof(io.dst.laddr);
-			/* LINTED pointer casts may be troublesome */
-			if (getsockname(out, (struct sockaddr *)&io.dst.laddr, &boundlen)
-			!= 0) {
+			if (getsockname(out, &io.dst.laddr, &boundlen) != 0) {
 				swarn("%s: getsockname(out)", function);
 				send_failure(request->s, &response, SOCKS_FAILURE);
 				close(request->s);
@@ -1095,16 +1080,19 @@ dorequest(mother, request)
 			}
 			/* remote out changes each time, set to INADDR_ANY for now. */
 			bzero(&io.dst.raddr, sizeof(io.dst.raddr));
-			io.dst.raddr.sin_family			= AF_INET;
-			io.dst.raddr.sin_addr.s_addr	= htonl(INADDR_ANY);
-			io.dst.raddr.sin_port			= htons(0);
+			/* LINTED pointer casts may be troublesome */
+			TOIN(&io.dst.raddr)->sin_family			= AF_INET;
+			/* LINTED pointer casts may be troublesome */
+			TOIN(&io.dst.raddr)->sin_addr.s_addr	= htonl(INADDR_ANY);
+			/* LINTED pointer casts may be troublesome */
+			TOIN(&io.dst.raddr)->sin_port				= htons(0);
 
 			if (request->req.flag & SOCKS_USECLIENTPORT)
-				if (client.sin_port == io.dst.laddr.sin_port)
+				/* LINTED pointer casts may be troublesome */
+				if (TOIN(&client)->sin_port == TOIN(&io.dst.laddr)->sin_port)
 					response.flag |= SOCKS_USECLIENTPORT;
 
-			/* LINTED pointer casts may be troublesome */
-			sockaddr2sockshost((struct sockaddr *)&io.src.laddr, &response.host);
+			sockaddr2sockshost(&io.src.laddr, &response.host);
 			response.reply	= (char)sockscode(response.version, SOCKS_SUCCESS);
 			flushio(mother, request->s, &response, &io);
 			break;
@@ -1307,10 +1295,9 @@ io_find(iolist, addr)
 
 	io = iolist;
 	while (io != NULL)
-		/* LINTED pointer casts may be troublesome */
-		if (sockaddrareeq((struct sockaddr *)&io->src.laddr, addr)
-		||  sockaddrareeq((struct sockaddr *)&io->dst.laddr, addr)
-		||  sockaddrareeq((struct sockaddr *)&io->control.laddr, addr))
+		if (sockaddrareeq(&io->src.laddr, addr)
+		||  sockaddrareeq(&io->dst.laddr, addr)
+		||  sockaddrareeq(&io->control.laddr, addr))
 			return io;
 		else
 			io = io->next;
