@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: sockd_util.c,v 1.66 2001/02/06 15:59:14 michaels Exp $";
+"$Id: sockd_util.c,v 1.72 2001/05/11 09:14:44 michaels Exp $";
 
 #define CM2IM(charmethodv, methodc, intmethodv) \
 	do { \
@@ -54,39 +54,57 @@ static const char rcsid[] =
 	} while (lintnoloop_sockd_h) \
 
 int
-selectmethod(methodv, methodc)
-	const unsigned char *methodv;
+selectmethod(methodv, methodc, offerdv, offeredc)
+	const int *methodv;
 	size_t methodc;
+	const unsigned char *offerdv;
+	size_t offeredc;
 {
-	const char stdmethodv[] = {AUTHMETHOD_NONE, AUTHMETHOD_UNAME};
-	int i;
+	size_t i;
+	size_t methodokc;
+	const char *methodokv;
 
-	for (i = 0; i < config.methodc; ++i) {
+	/* can select any standard method. */
+	const char rfc931methodv[] = {AUTHMETHOD_NONE, AUTHMETHOD_UNAME};
 
-		if (config.methodv[i] > AUTHMETHOD_NOACCEPT) { /* pseudo method */
-			int intmethodv[AUTHMETHOD_MAX];
+	/*
+	 * can select any standard method, some people want to use pam
+	 * without user/password.
+	*/
+	const char pammethodv[] = {AUTHMETHOD_UNAME, AUTHMETHOD_NONE};
 
-			CM2IM(methodv, methodc, intmethodv);
+	for (i = 0; i < methodc; ++i) {
+		if (methodv[i] > AUTHMETHOD_NOACCEPT) { /* non-socks method */
+			int intmethodv[MAXMETHOD];
+			size_t ii;
 
-			switch (config.methodv[i]) {
-				case AUTHMETHOD_RFC931: {
-					/* can select any standard method. */
-					size_t ii;
+			CM2IM(offerdv, offeredc, intmethodv);
 
-					for (ii = 0; ii < ELEMENTS(stdmethodv); ++i)
-						if (methodisset(stdmethodv[i], intmethodv, methodc))
-							return stdmethodv[i];
+			/* find the correct array to use for trying to find a ok method. */
+			switch (methodv[i]) {
+				case AUTHMETHOD_RFC931: 
+					methodokc = ELEMENTS(rfc931methodv);
+					methodokv = rfc931methodv;
+					break;
+
+				case AUTHMETHOD_PAM:
+					methodokc = ELEMENTS(pammethodv);
+					methodokv = pammethodv;
 					break;
-				}
 
 				default:
-					SERRX(config.methodv[i]);
+					SERRX(methodv[i]);
 			}
+
+			for (ii = 0; ii < methodokc; ++ii)
+				if (methodisset(methodokv[ii], intmethodv, offeredc))
+					return methodokv[ii];
+			
+			continue;
 		}
 
-		if (memchr(methodv, (unsigned char)config.methodv[i], (size_t)methodc)
-		!= NULL)
-			return config.methodv[i];
+		if (memchr(offerdv, (unsigned char)methodv[i], offeredc) != NULL)
+			return methodv[i];
 	}
 
 	return AUTHMETHOD_NOACCEPT;
@@ -274,40 +292,14 @@ socks_reseteuid(current, new)
 }
 
 int
-passwordcheck(name, clearpassword)
-	const char *name;
-	const char *clearpassword;
+pidismother(pid)
+	pid_t pid;
 {
-/*	const char *function = "passwordcheck()"; */
-	struct passwd *pw;
-	char *salt, *password;
-	uid_t euid;
-	int ok;
+	int i;
 
-	socks_seteuid(&euid, config.uid.privileged);
-	if ((pw = socks_getpwnam(name)) == NULL) {
-		salt		= "*";
-		password = "*";
-		ok			= 0;
-	}
-	else {
-		salt		= pw->pw_passwd;
-		password = pw->pw_passwd;
-		ok			= 1;
-	}
-	socks_reseteuid(config.uid.privileged, euid);
-
-	if (clearpassword != NULL) /* XXX waste cycles correctly? */
-		if (strcmp(crypt(clearpassword, salt), password) == 0)
-			ok = 1;
-		else
-			ok = 0;
-
-	if (ok)
-		return 0;
-
-	if (pw == NULL)
-		return 1;
-	else
-		return 2;
+	for (i = 0; i < config.option.serverc; ++i)
+		if (config.state.motherpidv[i] == pid)
+			return i + 1;
+	return 0;
 }
+
